<?php
	include("devices.php");

	function ResFromUA($useragent) {
		global $useragents, $devices;
		$res = array();

		while(list($agent, $viewmode) = each($useragents)) {
			if(strpos($useragent, $agent) !== false) {
				$count = count($viewmode);
				for($index = 0; $index < $count; $index++) {
					$res[] = $devices[$viewmode[$index]];
				}
			}
		}
		return($res);
	}

	function getString($string, $default) {
		if($string != "") {
			return $string;
		} else {
			return $default;
		}
	}

	function toUrl($param) {
		$parts = array();
		while(list($key, $val) = each($param)) {
			$parts[] = "$key=$val";
		}
		return implode("&amp;", $parts);
	}

	function cow($default, $overlay) {
		$out = $default;
		while(list($key, $val) = each($overlay)) {
			$out[$key] = $val;
		}
		return $out;
	}

	function getValueFromRequest($key, $default="") {
		if(isset($_REQUEST[$key])) {
			return $_REQUEST[$key];
		} else {
			return $default;
		}
	}

	function getValueFromCookie($key, $default="") {
		if(isset($_COOKIE[$key])) {
			return $_COOKIE[$key];
		} else {
			return $default;
		}
	}

	function getBugs($top, $bottom, $left, $right, $r=0) {
		$answer = file_get_contents("http://openstreetbugs.appspot.com/getBugs?b=$bottom&t=$top&l=$left&r=$right");
		$raw = split("\n", $answer);
		$bugs=array();
		while(list($key, $bug) = each($raw)) {
			$bug = substr_replace($bug, "", 0, strlen("putAJAXMarker("));
			$bug = substr_replace($bug, "", -2, strlen(");"));
			$flag = substr($bug, -1, 1);
			$bug = substr_replace($bug, "", -2, 2);
			$parts = explode(",", $bug, 4);
			if(isset($parts[3])) { // strip quotes if content field is non-empty
				$parts[3] = substr_replace($parts[3], "", 0, 1);
				$parts[3] = substr_replace($parts[3], "", -1, 1);
			}
			$parts[] = $flag;
			$parts["type"] = "bug";
			$bugs[$parts[0]] = $parts;
		}
		//OSB-Api might return just 10 Bugs
		//Limit recursion
		if(count($bugs) < 10 || $r > 5) {
			return $bugs;
		} else {
			$mh = ($top + $bottom)/2;
			$mw = ($left + $right)/2;
			return $bugs + getBugs($top, $mh, $left, $mw, $r+1) + getBugs($top, $mh, $mw, $right, $r+1) + getBugs($mh, $bottom, $left, $mw, $r+1) + getBugs($mh, $bottom, $mw, $right, $r+1);
		}
	}

	function getLinks($top, $bottom, $left, $right) {
		$points = array();
		$dom = new DOMDocument;
		if(@$dom->load("http://www.informationfreeway.org/api/0.5/node[website|url|link=*][bbox={$left},{$bottom},{$right},{$top}]")) {
			$xpath = new DOMXPath($dom);

			$nodes = $xpath->query('/osm/node');
			foreach($nodes as $node) {
				$lat =  getAttributeFromElement($node, "lat");
				$lon =  getAttributeFromElement($node, "lon");

				$tags =  $xpath->query('./tag', $node);
				$name = "";
				$link = "";
				foreach($tags as $tag) {
					$key = getAttributeFromElement($tag, "k");
					$value = getAttributeFromElement($tag, "v");

					if($key == "name") {
						$name = htmlentities(html_entity_decode($value, ENT_COMPAT, "UTF-8") , ENT_NOQUOTES, "UTF-8");
					} else if($key == "website" || $key == "url" || $key == "link") {
						if(strpos($value, "://") === false) {
							$link = "http://".$value;
						} else {
							$link = $value;
						}
					}
				}
				$point = array(0 => $name, 1 => $lon, 2 => $lat, 3 => $link, "type" => "link");
				$points[] = $point;
			}
		}
		return $points;
	}

	function getTileX($lon, $zoom) {
		return (($lon + 180) / 360) * pow(2, $zoom);
	}

	function getTileY($lat, $zoom) {
		return (1 - (log(tan($lat * pi() / 180) + 1/cos($lat * pi() / 180)) / pi())) / 2 * pow(2, $zoom);
	}

	function clipZoom($min, $zoom, $max) {
		if($zoom < $min) {
			return $min;
		} else if ($zoom > $max) {
			return $max;
		} else {
			return $zoom;
		}
	}

	function getRes() {
		//default
		$res = "320x240x3";
		$src = "default values";
		$options = array($res);

		// UA
		$reslist = ResFromUA($_SERVER["HTTP_USER_AGENT"]);
		if(count($reslist) != 0) {
			$res = $reslist[0];
			$options = array_merge($options, $reslist);
			$src = "the useragent";
		}

		// cookie
		$cookie_res = getValueFromCookie("res");
		if($cookie_res != "") {
			$res = $cookie_res;
			$options = array_merge($options, array($cookie_res));
			$src = "the cookie";
		}

		// parameter
		$width = getValueFromRequest("width", 0);
		$height = getValueFromRequest("height", 0);
		$border = getValueFromRequest("border", 0);
		if($width != 0 && $height != 0 && $border != 0) {
			$res = "{$width}x{$height}x{$border}";
			$options = array_merge($options, array($res));
			$src = "the query parameters";
		}
		$parts = split("x", $res);
		$out = array("val" => $res, "src" => $src, "options" => array_unique($options), "width" => $parts[0], "height" => $parts[1], "border" => $parts[2]);
		return $out;
	}


	function layerToString($layerArray) {
		$baselayers = array(0 => "tile", 1 => "tah", 2 => "cycle", "3" => "landsat");
		$overlays = array(0 => "maplint", 1 => "osb", 2 => "links");
		$layerstring = "";

		reset($baselayers);
		while(list($key, $base) = each($baselayers)) {
			if($layerArray["base"] == $base) {
				$layerstring .= "B";
			} else {
				$layerstring .= "0";
			}
		}

		reset($overlays);
		while(list($key, $overlay) = each($overlays)) {
			if($layerArray[$overlay] == true) {
				$layerstring .= "T";
			} else {
				$layerstring .= "F";
			}
		}

		return $layerstring;
	}

	function parseLayer($layerstring) {
		$baselayers = array(0 => "tile", 1 => "tah", 2 => "cycle", "3" => "landsat");
		$overlays = array(0 => "maplint", 1 => "osb", 2 => "links");
		$layerArray = array("base" => "tile");

		//$overlaystart = min(strpos($layerstring, "T"), strpos($layerstring, "F"));
		$overlaystart = strlen($layerstring);
		if(strpos($layerstring, "T") !== FALSE && strpos($layerstring, "T") < $overlaystart) {
			$overlaystart = strpos($layerstring, "T");
		}
		if(strpos($layerstring, "F") !== FALSE && strpos($layerstring, "F") < $overlaystart) {
			$overlaystart = strpos($layerstring, "F");
		}

		$baselayerstring = substr($layerstring, 0, $overlaystart);
		$overlaystring = substr($layerstring, $overlaystart);

		reset($baselayers);
		while(list($key, $base) = each($baselayers)) {
			if(isset($baselayerstring[$key]) && $baselayerstring[$key] == "B") {
				$layerArray["base"] = $base;
			}
		}

		reset($overlays);
		while(list($key, $overlay) = each($overlays)) {
			if(isset($overlaystring[$key]) && $overlaystring[$key] == "T") {
				$layerArray[$overlay] = true;
			} else {
				$layerArray[$overlay] = false;
			}
		}

		return $layerArray;
	}

	function getAttributeFromElement($element, $attribute) {
		$tmp = $element->attributes->getNamedItem($attribute);
		if($tmp != NULL) {
			return $tmp->nodeValue;
		} else {
			return "";
		}
	}
?>
